// This file is part of VKF, a CPython library
// for Machine Learning based on Lattice Theory.
//
// This Source Code Form is subject to the terms of the X11 (MIT) License.
// If a copy of the X11 License was not distributed with this file, 
// You can obtain one at https://opensource.org/licenses/MIT/.
//
// X11 (MIT) License
// 
// Copyright (c) 2020 Dmitry V. Vinogradov
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

#ifndef VKF_FRAGMENT_HPP
#define VKF_FRAGMENT_HPP

#include <cstdlib> // for std namespace
#include <fstream> // for std::ifstream, std::ofstream
#include <ios> // for std::ios::eof() function
#include <iostream> // for std::cout, std::cerr
#include <list> // for std::list
#include <memory> // for std::unique_ptr
#include <random> // for std::mt19937_64, std::uniform_int_distribution<>
#include <string> // for std::string
#include <utility> // for std::move()
#include <vector> // for std::vector

#include <boost/config.hpp> // for BOOST_SYMBOL_EXPORT
#include <boost/dll.hpp> // for boost::dll::import_alias, BOOST_DLL_ALIAS :error with MinGW64
#include <boost/dll/import.hpp> // for boost::dll::import
#include <boost/dynamic_bitset.hpp>

#ifndef API
// `extern "C"` - specifies C linkage: forces the compiler to export function/variable by a pretty (unmangled) C name
#define API extern "C" BOOST_SYMBOL_EXPORT
#endif //API

namespace vkf {
//    class BOOST_SYMBOL_EXPORT Fragment: public boost::dynamic_bitset<boost::dynamic_bitset<>::block_type> {
    class Fragment: public boost::dynamic_bitset<boost::dynamic_bitset<>::block_type> {
        protected:
        private:
        public:
            Fragment() = default;
            // Fragment constructor from move-constructor of base class
            Fragment(boost::dynamic_bitset<boost::dynamic_bitset<>::block_type>&& dbs) :
                boost::dynamic_bitset<boost::dynamic_bitset<>::block_type>(dbs) {}
    };

    typedef std::unique_ptr<Fragment> FragmentPtr;
} // namespace vkf

#endif // VKF_FRAGMENT_HPP
